using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using C1.C1Preview;

namespace Sample_ImageRendering
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        #region ***Events
        private void Form1_Load(object sender, EventArgs e)
        {
            //Binding C1FlexGrid
            categoriesTableAdapter.Connection = new OleDbConnection(GetConnectionString()); 
            categoriesTableAdapter.Fill(this.c1NWindDataSet.Categories);
            _flex.DataSource = c1NWindDataSet.Tables[0];
            
            //Displaying Picture Column and Attaching a OwnerDrawCell Event to _flex
            _flex.Cols["Picture"].Visible = true;
            _flex.DrawMode = C1.Win.C1FlexGrid.DrawModeEnum.OwnerDraw;
            _flex.OwnerDrawCell +=new C1.Win.C1FlexGrid.OwnerDrawCellEventHandler(_flex_OwnerDrawCell);
            
            // initialize styles
            _flex.Styles.Normal.Margins.Left = _flex.Styles.Normal.Margins.Right = 0;
            _flex.Styles.Normal.TextAlign = C1.Win.C1FlexGrid.TextAlignEnum.LeftTop;
            _flex.Styles.Fixed.TextAlign = C1.Win.C1FlexGrid.TextAlignEnum.LeftCenter;

            // make rows taller to show images
            int hei = _flex.Rows.DefaultSize;
            _flex.Rows.DefaultSize = hei * 4;
            _flex.Rows[0].Height = hei;
        }

        //Handles Custom Cell Drawing to display bound images
        private void _flex_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
        {
            // "Picture" is an image stored in a blob (byte[])
            if (_flex.Cols[e.Col].Name == "Picture")
            {
                // try loading from mdb
                e.Image = LoadImage(_flex[e.Row, e.Col] as byte[]);

                // if we got an image, blank text
                if (e.Image != null) e.Text = null;
            }
        }

        //Opens Form containing C1PrintPreviewControl displaying C1Flexgrid
        private void button1_Click(object sender, EventArgs e)
        {
            PreviewForm frm = new PreviewForm(GetControlImage(_flex, 1));
            frm.ShowDialog();
        }
        #endregion

        #region ***Methods
        // Method to convert C1FlexGrid into an image
        static Image GetControlImage(Control ctl, float zoom)
        {
            // get image
            Rectangle rc = ctl.ClientRectangle;
            Bitmap bmp = new Bitmap(rc.Width, rc.Height);
            ctl.DrawToBitmap(bmp, rc);

            // apply zoom
            if (zoom != 1)
            {
                Size newSize = Size.Round(new SizeF(bmp.Width * zoom, bmp.Height * zoom));
                bmp = new Bitmap(bmp, newSize);
            }

            // return image
            return bmp;
        }

        // load bitmap image stored in blob DB field
        // this assumes the image was stored as an OLE "package"
        // (this is the format used by Access)
        static Image LoadImage(byte[] picData)
        {
            // make sure this is an embedded object
            const int bmData = 78;
            if (picData == null || picData.Length < bmData + 2) return null;
            if (picData[0] != 0x15 || picData[1] != 0x1c) return null;

            // we only handle bitmaps for now
            if (picData[bmData] != 'B' || picData[bmData + 1] != 'M') return null;

            // load the picture
            Image img = null;
            try
            {
                MemoryStream ms = new MemoryStream(picData, bmData, picData.Length - bmData);
                img = Image.FromStream(ms);
            }
            catch { }

            // return what we got
            return img;
        }

        //Returns the connection string for the datasource
        static string GetConnectionString()
        {
            string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\ComponentOne Samples\Common";
            string conn = @"provider=microsoft.jet.oledb.4.0;data source={0}\c1nwind.mdb;";
            return string.Format(conn, path);
        }
        #endregion
    }
}